import java.io.*;
import java.util.*;

/**
  * FormCData	Class
  */


public class FormCData { 
	
	FormCHtml fch = new FormCHtml(); 	
	FormCDataParser fc;
	
	String formActionURL;
	
	int errorCode;
	String fieldNames[];
	String fieldValues[];  
	
	Properties fileProp;
	
	static final int ERROR_PROPERTY_FILE = 0;
	static final int ERROR_CSV_FILE      = 1;
	static final int ERROR_UNKNOWN       = 2;
	static final int ERROR_SELECTION     = 3;
	
 /**
   * init() 
   *
   * @return        String		    html
   */

	public boolean init() {
		
		try {
			fileProp = FileUtil.loadProperties("FormC.Property");
			File file = FileUtil.getFile(fileProp.getProperty("FileName"));		
			formActionURL = fileProp.getProperty("ActionURL");		
			fc  = new FormCDataParser(FileUtil.openInputFile(file));
			return true;
		}
		catch (PropertyFileNotFoundException pfnf) {
			errorCode = ERROR_PROPERTY_FILE;			
			return false;
		}	
		catch (FileNotFoundException fnf) {
			errorCode = ERROR_CSV_FILE;			
			return false;
		}	
		catch (IOException io) {
			errorCode = ERROR_UNKNOWN;			
			return false;
		}
	}	
	
 /**
   * getDefaultFormC()
   *
   * @return        String		    html
   */

	public String getDefaultFormC() {
	
		return(fch.formCSelect(fc.getTerm(), fc.getCourse(),
					           fc.getSection(), formActionURL)); 
		
	}
	
 /**
   * getErrorFormC() 
   *
   * @return        String		    html
   */

	public String getErrorFormC() {
	
		return(fch.formCError(errorCode)); 
		
	}

 /**
   * getSelectFormC() 
   *
   * @parm 			String			selectedTerm
   * @parm 			String			selectedCourse
   * @parm 			String			selectedInstructor
   * @return        String		    html
   */

	public String getSelectFormC(String selTerm, String selCourse, 
											  		 String selSection) {
					
		StringBuffer selection = new StringBuffer();
		
		selection.append(selTerm);
		selection.append(" ");
		selection.append(selCourse);
		selection.append(" ");
		selection.append(selSection);
		
		String selInstructor = null;
		String selStudents[] = null;
				
		if (fc.isCourseKey(selection.toString())) {
			int keyIndex  = fc.getCourseKeyIndex();	
			selInstructor = fc.getCourseInstructorByKey(keyIndex);
			selStudents   = fc.getCourseStudentsByKey(keyIndex);

 			return(fch.formC(selTerm, selCourse, selInstructor, 
 							          selStudents, formActionURL));
        }
		else {
		
	   		return (fch.formCError(ERROR_SELECTION));	
	   	}	
		
	}	

	
	
 /**
   * getDisplayFormC() 
   *
   * @parm          String[]        fNames
   * @parm          String[]        fValues 
   * @return        String		    html
   */
	
	public String getDisplayFormC(String [] fNames, 
								  String [] fValues) {

		int keyIndex  = fc.getCourseKeyIndex();	
		
		fieldNames = new String[fNames.length];
		
		for (int i = 0; i < fNames.length; i++) {
			fieldNames[i] = fNames[i];
		}
			
		fieldValues = new String[fValues.length];
		
		for (int i = 0; i < fValues.length; i++) {
			fieldValues[i] = fValues[i];
		}
			
		return(fch.formCData(fc.getCourseTermByKey(keyIndex),
							 fc.getCourseNoByKey(keyIndex) + " " +
							    fc.getCourseNameByKey(keyIndex),
							 fc.getCourseInstructorByKey(keyIndex),
							 fc.getCourseStudentsByKey(keyIndex),
					         fieldNames, 
					         fieldValues));
   		
	
	}
	
  /**
   * getSqlFormC() 
   *
   * @return        String		    html
   */
	
	public String getSqlFormC()  {

		FormCSQL fcSql = new FormCSQL(fc);
		return fcSql.generateFormCSQL(fieldNames, fieldValues);

	}
}
