/*
 * $Id: AccountDAO.java,v 1.8 2000/10/04 05:11:27 brydon Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */

package com.sun.j2ee.blueprints.customer.account.ejb;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.sql.DataSource;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.ejb.FinderException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.CreateException;
import javax.ejb.RemoveException;

import com.sun.j2ee.blueprints.customer.util.JNDINames;
import com.sun.j2ee.blueprints.customer.account.ejb.AccountDAOException;
import com.sun.j2ee.blueprints.customer.util.Debug;
import com.sun.j2ee.blueprints.customer.util.DatabaseNames;
import com.sun.j2ee.blueprints.customer.util.ContactInformation;
import com.sun.j2ee.blueprints.customer.util.Address;

/**
 * This class encapsulates all the JDBC calls made by the AccountEJB.
 * Actual logic of inserting/fetching/updating/deleting  the data in
 * relational database tables to mirror the state of AccountEJB is
 * implemented here.
 */
public class AccountDAO {

    private String userId;
    private String status;
    private ContactInformation info;

    private Connection dbConnection = null;
    private DataSource datasource   = null;

    public AccountDAO() throws AccountDAOException {
        try {
            InitialContext ic = new InitialContext();
            datasource = (DataSource) ic.lookup(JNDINames.ESTORE_DATASOURCE);
        } catch (NamingException ne) {
            throw new AccountDAOException("Naming Exception while looking " +
                                               " up DataSource Connection " +
                                                JNDINames.ESTORE_DATASOURCE +
                                                    ": \n" + ne.getMessage());
        }
    }

    // get and set methods for the instance variables
    public String getUserId(){
        return userId;
    }

    public String getStatus() {
        return status;
    }

    public ContactInformation getContactInformation() {
        return info;
    }

    public void setUserId( String userId) {
        this.userId = userId;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public void setContactInformation(ContactInformation info) {
        this.info = info;
    }

    public void create() throws AccountDAOException {
        insertAccount();
    }

    public void load() throws AccountDAOException {
        selectAccount();
    }

    public void store() throws AccountDAOException {
        updateAccount();
    }

    public void remove() throws AccountDAOException {
        deleteAccount();
    }

    public String findByPrimaryKey() throws AccountDAOException {
        if (userExists(userId))
            return (userId);
        throw new AccountDAOException("primary key not found :" + userId);
    }

    private boolean userExists (String userId) throws AccountDAOException {
        String queryStr ="SELECT userid FROM " +
                    DatabaseNames.ACCOUNT_TABLE
                        + " WHERE userid = " + "'" + userId.trim() + "'";
        Debug.println("queryString is: "+ queryStr);

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            ResultSet result = stmt.executeQuery(queryStr);
            if ( !result.next() ) {
                return false;
            } else {
                userId = result.getString(1);
                return true;
            }
        } catch(SQLException se) {
            throw new AccountDAOException("SQLException while checking for an"
                            + " existing user - id -> " + userId + " :\n" + se);
        } finally {
            closeConnection();
        }
    }

    private boolean isValidData () {
        if ( (userId == null) ||
             ( info.getEMail() == null) ||
             (info.getGivenName() == null) || (info.getFamilyName() == null)
             || (info.getAddress().getStreetName1() == null) ||
             (info.getAddress().getCity() == null) ||
             (info.getAddress().getState() == null) ||
             (info.getAddress().getZipCode() == null) ||
             (info.getAddress().getCountry() == null)
             || (info.getTelephone() == null) )
            return (false);
        else
            return (true);
    }

    private void insertAccount() throws AccountDAOException {

        if ( !isValidData() )
            throw new AccountDAOException("Illegal data values for insert");
        if (userExists(userId))
            throw new AccountDAOException("Account Exists:   " + userId );

        String queryStr = "INSERT INTO " + DatabaseNames.ACCOUNT_TABLE +
            "(userid,email,firstname,lastname,status,"
            + "addr1,addr2,city,state,zip,country,"
            + "phone)" + "VALUES ("
            + "'" + userId.trim() + "',"
            + "'" + info.getEMail().trim() + "',"
            + "'" + info.getGivenName().trim() + "',"
            + "'" + info.getFamilyName().trim() + "',"
            + "'" + status.trim() + "',"
            + "'" + info.getAddress().getStreetName1().trim() +"',";

        if (info.getAddress().getStreetName2() != null)
            queryStr += "'"+info.getAddress().getStreetName2().trim() +"',";
        else
            queryStr += "' ',";

        queryStr +=  "'" + info.getAddress().getCity().trim() + "',"
            + "'" + info.getAddress().getState().trim() + "',"
            + "'" + info.getAddress().getZipCode().trim() + "',"
            + "'" + info.getAddress().getCountry().trim() + "',"
            + "'" + info.getTelephone().trim() + "' )";

        Debug.println("queryString is: "+ queryStr);

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            int resultCount = stmt.executeUpdate(queryStr);

            if ( resultCount != 1 )
                throw new AccountDAOException(
                    "ERROR in ACCOUNT_TABLE INSERT !! resultCount = " +
                                   resultCount);
        } catch(SQLException ae) {
            throw new AccountDAOException("SQLException while inserting new " +
                            "account; id = " + userId + " :\n" + ae);
        } finally {
            closeConnection();
        }
    }

    private void selectAccount () throws AccountDAOException {

        String queryStr = "SELECT "+
            "userid,status,email,firstname,lastname,"+
                "addr1,addr2,city,state,zip,country,phone"+
                    " FROM " + DatabaseNames.ACCOUNT_TABLE +
                        " WHERE userid = " + "'" + userId.trim() + "'";
        Debug.println("queryString is: "+ queryStr);

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            ResultSet result = stmt.executeQuery(queryStr);

            if ( !result.next() )
                throw new AccountDAOException("No record for primary key " +
                                   userId);

            int i = 1;
            userId = result.getString(i++);
            status = result.getString(i++);
            String email = result.getString(i++);
            String firstName = result.getString(i++);
            String lastName = result.getString(i++);
            String street1 = result.getString(i++);
            String street2 = result.getString(i++);
            String city = result.getString(i++);
            String state = result.getString(i++);
            String zip = result.getString(i++);
            String country = result.getString(i++);
            String phone = result.getString(i++);

            Address addr = new Address(street1, street2, city, state, zip,
                               country);
            info = new ContactInformation(lastName, firstName, phone,
                              email, addr);
        } catch(SQLException ae) {
            throw new AccountDAOException("SQLException while getting " +
                            "account; id = " + userId + " :\n" + ae);
        } finally {
            closeConnection();
        }
    }

    private void deleteAccount () throws AccountDAOException {
        String queryStr = "DELETE FROM " + DatabaseNames.ACCOUNT_TABLE
        + " WHERE userid = " + "'" + userId.trim() + "'";
        Debug.println("queryString is: "+ queryStr);

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            int resultCount = stmt.executeUpdate(queryStr);

            if (resultCount != 1)
                throw new AccountDAOException
                ("ERROR deleteing account from ACCOUNT_TABLE!! resultCount = " +
                 resultCount);
        } catch(SQLException se) {
            throw new AccountDAOException("SQLException while removing " +
                            "account; id = " + userId + " :\n" + se);
        } finally {
            closeConnection();
        }
    }

    private void updateAccount () throws AccountDAOException {
        if (!isValidData())
            throw new AccountDAOException("Illegal data values for update");
        String queryStr = "UPDATE " + DatabaseNames.ACCOUNT_TABLE + " SET "
            + "status = " + "'" + status.trim() + "',"
            + "email = " + "'" + info.getEMail().trim() + "',"
            + "firstname = " + "'" + info.getGivenName().trim() + "',"
            + "lastname = " + "'" + info.getFamilyName().trim() + "',"
            + "addr1 = " + "'"
            + info.getAddress().getStreetName1().trim() + "',";
        if (info.getAddress().getStreetName2() != null)
            queryStr += " addr2 = " + "'"
                +info.getAddress().getStreetName2().trim() +"',";
        else
            queryStr += " addr2 = " + "' ',";
            queryStr +=  "city = " + "'" + info.getAddress().getCity().trim() + "',"
            + "state = " + "'" + info.getAddress().getState().trim() + "',"
            + "zip = " + "'" + info.getAddress().getZipCode().trim() + "',"
            + "country = " + "'" + info.getAddress().getCountry().trim() + "',"
            + "phone = " + "'" + info.getTelephone().trim() + "'"
            + " WHERE userid = " + "'" + userId.trim() + "'";
        Debug.println("queryString is: "+ queryStr);

        try {
            getDBConnection();
            Statement stmt = dbConnection.createStatement();
            int resultCount = stmt.executeUpdate(queryStr);

            if (resultCount != 1)
                throw new AccountDAOException
                ("ERROR updating account in ACCOUNT_TABLE!! resultCount = " +
                 resultCount);
        } catch(SQLException se) {
            throw new AccountDAOException("SQLException while updating " +
                            "account; id = " + userId + " :\n" + se);
        } finally {
            closeConnection();
        }
    }

    private void getDBConnection() throws AccountDAOException {
        try {
            dbConnection = datasource.getConnection();
        } catch (SQLException se) {
            throw new AccountDAOException("SQL Exception while getting " +
                                "DB connection : \n" + se);
        }
        return;
    }

    private void closeConnection() throws AccountDAOException {
        try {
            if (dbConnection != null && !dbConnection.isClosed()) {
                dbConnection.close();
            }
        } catch (SQLException se) {
            throw new AccountDAOException("SQL Exception while closing " +
                                        "DB connection : \n" + se);
        }
    }

}
