/*
 * $Id: AccountEJB.java,v 1.9 2000/10/05 04:38:00 inder Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */

package com.sun.j2ee.blueprints.customer.account.ejb;

import java.rmi.RemoteException;

import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import javax.ejb.DuplicateKeyException;

import com.sun.j2ee.blueprints.customer.account.ejb.AccountDAOException;
import com.sun.j2ee.blueprints.customer.util.ContactInformation;
import com.sun.j2ee.blueprints.customer.account.model.AccountModel;
import com.sun.j2ee.blueprints.customer.util.Debug;

/**
 * Implementation of account as an Entity Bean
 */
public class AccountEJB implements EntityBean {

    private String userId;
    private String status;
    private ContactInformation info;

    private EntityContext context;
    private AccountDAO accountDao;

    public AccountEJB() {}

    public String ejbCreate (String userId, String status,
                             ContactInformation info)
                throws DuplicateKeyException,CreateException {

        // set the instance data
        this.userId = userId;
        this.status = status;
        this.info = info;

        try {
            AccountDAO dao = getDAO();
            dao.setUserId(userId);
            dao.setStatus(status);
            dao.setContactInformation(info);
            dao.create();
            return (userId);
        } catch(AccountDAOException se) {
            context.setRollbackOnly();
            throw new CreateException (se.getMessage());
        }
    }

    public void ejbRemove() throws RemoveException {
        try{
            AccountDAO dao = getDAO();
            dao.setUserId(this.userId);
            dao.remove();
        } catch (AccountDAOException se) {
            context.setRollbackOnly();
            throw new RemoveException (se.getMessage());
        }
    }

    public void setEntityContext(EntityContext ec) {
        context = ec;
    }

    public void ejbLoad() {
        AccountDAO dao;
        try{
            dao = getDAO();
            dao.setUserId(this.userId);
            dao.load();
        } catch (AccountDAOException se) {
            throw new EJBException (se.getMessage());
        }
        userId = dao.getUserId();
        status = dao.getStatus();
        info = dao.getContactInformation();
    }

    public void ejbStore() {
        try{
            AccountDAO dao = getDAO();
            dao.setUserId(this.userId);
            dao.setStatus(status);
            dao.setContactInformation(info);
            dao.store();
        } catch (AccountDAOException se) {
            throw new EJBException (se.getMessage());
        }
    }

    public String ejbFindByPrimaryKey (String key) throws FinderException {
        try{
            AccountDAO dao = getDAO();
            dao.setUserId(key);
            return(dao.findByPrimaryKey());
        } catch (AccountDAOException se) {
            throw new FinderException (se.getMessage());
        }
    }

    public void unsetEntityContext() {}

    public void ejbActivate() {
      this.userId = (String)context.getPrimaryKey();
    }

    public void ejbPassivate() {
      this.userId     = null;
      this.accountDao = null;
    }

    public void ejbPostCreate(String userId,String status,
                              ContactInformation info)
        throws DuplicateKeyException,CreateException {
    }

    // business methods

    public void changeContactInformation(ContactInformation info) {
        this.info = info;
    }

    public AccountModel getDetails() {
        return(new AccountModel(userId, status, info));
    }


    private AccountDAO getDAO() throws AccountDAOException {
        if(accountDao == null) {
           accountDao = new AccountDAO();
        }
        return accountDao;
    }
}
